SystemOrganization addCategory: #VersionChangesCalculator!

Object subclass: #ChangeCriteria
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

ChangeCriteria subclass: #ChangeByCyclomaticComplexity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!ChangeByCyclomaticComplexity methodsFor: 'as yet unclassified' stamp: ''!
getNumber: aClass 
	^ aClass weightedMethodCount! !

ChangeCriteria subclass: #ChangeByLinesNum
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!ChangeByLinesNum methodsFor: 'as yet unclassified' stamp: ''!
getNumber: aClass 
	^ aClass numberOfLinesOfCode! !

ChangeCriteria subclass: #ChangeByMethodsNum
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!ChangeByMethodsNum methodsFor: 'as yet unclassified' stamp: ''!
getNumber: aClass 
	^ aClass numberOfMethods! !

!ChangeCriteria methodsFor: 'as yet unclassified' stamp: ''!
getNumber: aClass 
	^ 0! !

Object subclass: #ClassesInfoManager
	instanceVariableNames: 'classesInfo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!ClassesInfoManager methodsFor: 'as yet unclassified' stamp: ''!
addAllClassesInfo: classesNewInfo 
	classesNewInfo
		keysDo: [:className | self
				addVersionsInfo: (classesNewInfo at: className)
				forClass: className].
	^ classesInfo! !

!ClassesInfoManager methodsFor: 'as yet unclassified' stamp: ''!
addVersionsInfo: versionsInfo forClass: className 
	(classesInfo includesKey: className)
		ifTrue: [(classesInfo at: className)
				addAll: versionsInfo]
		ifFalse: [classesInfo at: className put: versionsInfo]! !

!ClassesInfoManager methodsFor: 'as yet unclassified' stamp: ''!
classesInfo
	^ classesInfo! !

!ClassesInfoManager methodsFor: 'as yet unclassified' stamp: ''!
getAllExistingVersions
	| versionNums |
	versionNums := SortedCollection new.
	classesInfo
		valuesDo: [:classVersionsInfo | classVersionsInfo allVersionNums
				do: [:verNum | (versionNums includes: verNum)
						ifFalse: [versionNums add: verNum]]].
	^ versionNums! !

!ClassesInfoManager methodsFor: 'as yet unclassified' stamp: ''!
getChangedClassesFromVersion: beginVersion toVersion: endVersion 
	| changedClasses |
	changedClasses := LinkedList new.
	classesInfo
		keysDo: [:className | 
			| classVersionsInfo beginVersionInfo endVersionInfo |
			classVersionsInfo := classesInfo at: className.
			beginVersionInfo := classVersionsInfo getInfoForVersion: beginVersion.
			endVersionInfo := classVersionsInfo getInfoForVersion: endVersion.
			beginVersionInfo isNil not & (beginVersionInfo ~= endVersionInfo)
				ifTrue: [changedClasses add: className]].
	^ changedClasses! !

!ClassesInfoManager methodsFor: 'as yet unclassified' stamp: ''!
getClassesInfoFromVersion: beginVersion toVersion: endVersion 
	| windowClassesInfo |
	windowClassesInfo := ClassesInfoManager new.
	classesInfo
		keysDo: [:className | 
			| windowClassVersionsInfo |
			windowClassVersionsInfo := (classesInfo at: className)
						getInfoFromVersion: beginVersion
						toVersion: endVersion.
			windowClassVersionsInfo isEmpty
				ifFalse: [windowClassesInfo addVersionsInfo: windowClassVersionsInfo forClass: className]].
	^ windowClassesInfo! !

!ClassesInfoManager methodsFor: 'as yet unclassified' stamp: ''!
initialize
	classesInfo := Dictionary new! !

!ClassesInfoManager methodsFor: 'as yet unclassified' stamp: ''!
lastExistingVersion
	| lastVersion |
	lastVersion := 0.
	classesInfo
		valuesDo: [:classVersionsInfo | 
			| versionNum |
			versionNum := classVersionsInfo last versionNum.
			versionNum asInteger > lastVersion
				ifTrue: [lastVersion := versionNum]].
	^ lastVersion! !

!ClassesInfoManager methodsFor: 'as yet unclassified' stamp: ''!
loadClassesInfoFromFile: critFileName 
	| stream lineAsArray className versionsInfo |
	stream := FileStream readOnlyFileNamed: critFileName.
	Transcript show: 'Adding classes info from file ' , critFileName , ' ...';
		 cr;
		 cr.
	[stream atEnd]
		whileFalse: [lineAsArray := stream nextLine findBetweenSubStrs: '='.
			className := lineAsArray at: 1.
			versionsInfo := VersionsList new.
			((lineAsArray at: 2)
				findBetweenSubStrs: ',')
				do: [:value | 
					| splitValue versInfo |
					splitValue := value findBetweenSubStrs: '-'.
					versInfo := VersionInfo new.
					versInfo versionNum: (splitValue at: 1) asInteger.
					versInfo
						value: (splitValue at: 2).
					versionsInfo add: versInfo].
			classesInfo at: className put: versionsInfo].
	stream close! !

!ClassesInfoManager methodsFor: 'as yet unclassified' stamp: ''!
removeDeprecatedClasses
	| lastVersionInfo |
	lastVersionInfo := VersionInfo new.
	lastVersionInfo versionNum: self lastExistingVersion.
	classesInfo
		keysDo: [:className | ((classesInfo at: className)
					includes: lastVersionInfo)
				ifFalse: [classesInfo removeKey: className]].
	^ classesInfo! !

!ClassesInfoManager methodsFor: 'as yet unclassified' stamp: ''!
saveClassesInfoToFile: fileName 
	| stream versionsInfo |
	stream := FileStream forceNewFileNamed: fileName.
	Transcript show: 'Saving classes info to file ' , fileName , ' ...';
		 cr;
		 cr.
	classesInfo
		keysDo: [:className | 
			stream nextPutAll: className , '='.
			versionsInfo := classesInfo at: className.
			versionsInfo
				do: [:versInfo | 
					stream nextPutAll: versInfo versionNum asString , '-' , versInfo value asString.
					versionsInfo last = versInfo
						ifFalse: [stream nextPutAll: ',']].
			stream nextPutAll: '';
				 cr].
	stream close! !

Object subclass: #MovingAverageCalculator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!MovingAverageCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesMovingAverages: classesDict withPeriod1: period1 andPeriod2: period2 saveToFile: fileName 
	| stream movAvg1 movAvg2 |
	stream := FileStream forceNewFileNamed: fileName.
	Transcript show: 'Calculating Moving Averages...';
		 cr;
		 cr.
	classesDict
		keysDo: [:className | 
			| classValues versionNumsList valuesList |
			stream nextPutAll: 'Class Name: ' , className;
				 cr.
			classValues := classesDict at: className.
			versionNumsList := LinkedList new.
			valuesList := LinkedList new.
			classValues
				do: [:versionInfo | 
					versionNumsList add: versionInfo versionNum.
					valuesList add: versionInfo value].
			stream nextPutAll: 'Version Nums: , ' , versionNumsList asCommaString;
				 cr.
			stream nextPutAll: 'Values: , ' , valuesList asCommaString;
				 cr.
			movAvg1 := self calculateMovingAverage: classValues withPeriod: period1.
			movAvg2 := self calculateMovingAverage: classValues withPeriod: period2.
			stream nextPutAll: 'Mov Avg ' , period1 asString , ': , ' , movAvg1 asCommaString;
				 cr.
			stream nextPutAll: 'Mov Avg ' , period2 asString , ': , ' , movAvg2 asCommaString;
				 cr;
				 cr].
	stream close! !

!MovingAverageCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesRangeReliance: classesCritDict withPeriod1: period1 andPeriod2: period2 
	| classesReliance |
	Transcript show: 'Calculating Classes Moving Average Reliance using Ranges...';
		 cr;
		 cr.
	classesReliance := OrderedCollection new.
	classesCritDict
		keysDo: [:className | 
			| classInfo |
			classInfo := ClassInfo new.
			classInfo className: className.
			classInfo
				value: (self
						calculateRangeReliance: (classesCritDict at: className)
						withPeriod1: period1
						andPeriod2: period2).
			classesReliance add: classInfo].
	^ classesReliance sort reverse! !

!MovingAverageCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesRangeReliance: classesCritDict withPeriod1: period1 andPeriod2: period2 saveToFile: fileName 
	| classesReliance |
	classesReliance := self
				calculateClassesRangeReliance: classesCritDict
				withPeriod1: period1
				andPeriod2: period2.
	^ self saveClassesReliance: classesReliance toFile: fileName! !

!MovingAverageCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesReliance: classesCritDict withPeriod1: period1 andPeriod2: period2 
	| classesReliance |
	Transcript show: 'Calculating Classes Moving Average Reliance...';
		 cr;
		 cr.
	classesReliance := OrderedCollection new.
	classesCritDict
		keysDo: [:className | 
			| classInfo |
			classInfo := ClassInfo new.
			classInfo className: className.
			classInfo
				value: (self
						calculateReliance: (classesCritDict at: className)
						withPeriod1: period1
						andPeriod2: period2).
			classesReliance add: classInfo].
	^ classesReliance sort reverse! !

!MovingAverageCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesReliance: classesCritDict withPeriod1: period1 andPeriod2: period2 saveToFile: fileName 
	| classesReliance |
	classesReliance := self
				calculateClassesReliance: classesCritDict
				withPeriod1: period1
				andPeriod2: period2.
	^ self saveClassesReliance: classesReliance toFile: fileName! !

!MovingAverageCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateMovingAverage: versionsInfo withPeriod: period 
	| movingAverages periodList mvaCount periodCount totalPeriodValue |
	periodCount := 1.
	totalPeriodValue := 0.
	periodList := LinkedList new: period.
	movingAverages := Array new: versionsInfo size.
	mvaCount := period.
	versionsInfo
		do: [:versInfo | 
			| value |
			value := versInfo value.
			periodCount < period
				ifTrue: [periodCount := periodCount + 1.
					value isNil
						ifFalse: [totalPeriodValue := totalPeriodValue + value].
					periodList add: value]
				ifFalse: [periodList add: value.
					value isNil
						ifFalse: [totalPeriodValue := totalPeriodValue + value.
							totalPeriodValue := totalPeriodValue - periodList removeFirst].
					movingAverages at: mvaCount put: (totalPeriodValue / period) asFloat.
					mvaCount := mvaCount + 1]].
	^ movingAverages! !

!MovingAverageCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateRangeReliance: values withPeriod1: period1 andPeriod2: period2 
	| mva1Values mva2Values crossOverCount hitsCount index slowerMVA fasterMVA reliance breakPoints crossoverPoints breakPoint1 breakPoint2 |
	breakPoints := self getBreakPoints: values.
	mva1Values := self calculateMovingAverage: values withPeriod: period1.
	mva2Values := self calculateMovingAverage: values withPeriod: period2.
	crossoverPoints := self getCrossoverPoints: mva1Values withMVA2: mva2Values.
	hitsCount := 0.
	index := 1.
	[index < breakPoints size]
		whileTrue: [breakPoint1 := breakPoints at: index.
			breakPoint2 := breakPoints at: index + 1.
			(self
					crossoverExists: crossoverPoints
					between: breakPoint1
					and: breakPoint2)
				ifTrue: [hitsCount := hitsCount + 1].
			index := index + 1].
	reliance := 0 asFloat.
	hitsCount > 0
		ifTrue: [reliance := (hitsCount / (breakPoints size - 1)) asFloat].
	^ reliance! !

!MovingAverageCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateReliance: values withPeriod1: period1 andPeriod2: period2 
	| mva1Values mva2Values crossOverCount hitsCount index slowerMVA fasterMVA reliance |
	mva1Values := self calculateMovingAverage: values withPeriod: period1.
	mva2Values := self calculateMovingAverage: values withPeriod: period2.
	crossOverCount := 0.
	hitsCount := 0.
	period1 > period2
		ifTrue: [index = period1 + 1.
			fasterMVA = mva2Values.
			slowerMVA = mva1Values]
		ifFalse: [index := period2 + 1.
			fasterMVA := mva1Values.
			slowerMVA := mva2Values].
	[index < values size]
		whileTrue: [(fasterMVA at: index - 1)
					> (slowerMVA at: index - 1) = ((fasterMVA at: index)
						> (slowerMVA at: index))
				ifFalse: [crossOverCount := crossOverCount + 1.
					(fasterMVA at: index)
							< (fasterMVA at: index - 1) & ((values at: index)
								< (values at: index + 1))
						ifTrue: [hitsCount := hitsCount + 1].
					(fasterMVA at: index)
							> (fasterMVA at: index - 1) & ((values at: index)
								> (values at: index + 1))
						ifTrue: [hitsCount := hitsCount + 1].
					(fasterMVA at: index)
							= (fasterMVA at: index - 1) & ((values at: index)
								= (values at: index + 1))
						ifTrue: [hitsCount := hitsCount + 1]].
			index := index + 1].
	reliance := 0 asFloat.
	crossOverCount > 0
		ifTrue: [reliance := (hitsCount / crossOverCount) asFloat].
	^ reliance! !

!MovingAverageCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateValuesDiff: value1 andValue2: value2 
	| valuesDiff |
	valuesDiff := value1 asInteger - value2 asInteger.
	valuesDiff < 0
		ifTrue: [^ 1].
	valuesDiff > 0
		ifTrue: [^ -1].
	^ 0! !

!MovingAverageCalculator methodsFor: 'as yet unclassified' stamp: ''!
createLineChart: values withMAValues: movAvgValues1 andMAValues2: movAvgValues2 
	| diag1 diag2 diag3 compDiag |
	diag1 := ESDiagramRenderer new lineDiagram y: #yourself;
				 models: values;
				 defaultColor: Color red;
				 yourself.
	diag2 := ESDiagramRenderer new lineDiagram y: #yourself;
				 defaultColor: Color green;
				 models: movAvgValues2;
				 yourself.
	diag3 := ESDiagramRenderer new lineDiagram y: #yourself;
				 defaultColor: Color blue;
				 models: movAvgValues1;
				 yourself.
	compDiag := ESDiagramRenderer new.
	compDiag compositeDiagram add: diag1;
		 add: diag2;
		 add: diag3;
		 preferredAxisMaxY: 45.
	^ compDiag! !

!MovingAverageCalculator methodsFor: 'as yet unclassified' stamp: ''!
crossoverExists: crossoverPoints between: value1 and: value2 
	crossoverPoints
		do: [:co | 
			co >= value1 & (co < value2)
				ifTrue: [^ true].
			nil].
	^ false! !

!MovingAverageCalculator methodsFor: 'as yet unclassified' stamp: ''!
getBreakPoints: values 
	| breakPoints index trendIndicator valuesDiff value1 value2 |
	breakPoints := LinkedList new.
	index := 1.
	trendIndicator := 10.
	[index < values size]
		whileTrue: [value1 := (values at: index) value.
			value2 := (values at: index + 1) value.
			valuesDiff := self calculateValuesDiff: value1 andValue2: value2.
			valuesDiff = trendIndicator
				ifFalse: [trendIndicator := valuesDiff.
					breakPoints add: index].
			index := index + 1].
	breakPoints add: values size.
	^ breakPoints! !

!MovingAverageCalculator methodsFor: 'as yet unclassified' stamp: ''!
getCrossoverPoints: mva1 withMVA2: mva2 
	| crossoverPoints mva1Value mva2Value |
	crossoverPoints := LinkedList new.
	1
		to: mva1 size
		do: [:index | (mva1 at: index)
					= (mva2 at: index)
				ifTrue: [crossoverPoints add: index]].
	^ crossoverPoints! !

!MovingAverageCalculator methodsFor: 'as yet unclassified' stamp: ''!
saveClassesReliance: classesReliance toFile: fileName 
	| stream |
	stream := FileStream forceNewFileNamed: fileName.
	stream nextPutAll: 'Classes Moving Average Reliance:';
		 cr;
		 cr.
	classesReliance
		do: [:classInfo | stream nextPutAll: classInfo className , ' = ' , classInfo value asString;
				 cr].
	stream close! !

Object subclass: #VersionChangesCalculator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!VersionChangesCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassVersionChanges: classHistory withCriteria: changeCriteria numberOfVersions: numberOfVersions 
	| versionsChangesArray versionNum |
	versionsChangesArray := Array new: numberOfVersions.
	versionNum := 1.
	classHistory versions
		do: [:classVersion | 
			| previousClassVersion diffValue value1 value2 |
			versionNum := versionNum + 1.
			previousClassVersion := classVersion previousVersion.
			previousClassVersion isNil
				ifTrue: [diffValue := changeCriteria getNumber: classVersion]
				ifFalse: [value1 := changeCriteria getNumber: classVersion.
					value2 := changeCriteria getNumber: previousClassVersion.
					diffValue := (value1 - value2) abs].
			versionNum > 1
				ifTrue: [versionsChangesArray at: versionNum - 1 put: diffValue]].
	^ versionsChangesArray! !

!VersionChangesCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesVersionChanges: hismoModel withCriteria: changeCriteria 
	| classesChangesDict classesHistories versionsNum |
	classesChangesDict := Dictionary new.
	classesHistories := hismoModel allClassHistories
				select: [:each | each last isStub not & each isRemoved not].
	versionsNum := hismoModel versions size.
	classesHistories
		do: [:classHistory | 
			| className changesArray |
			className := classHistory name.
			changesArray := self
						calculateClassVersionChanges: classHistory
						withCriteria: changeCriteria
						numberOfVersions: versionsNum.
			classesChangesDict at: className put: changesArray].
	^ classesChangesDict! !

!VersionChangesCalculator methodsFor: 'as yet unclassified' stamp: ''!
createClassChangesLineChart: changesArray 
	| chart |
	chart := ESDiagramRenderer new.
	chart lineDiagram y: #changeNum;
		 yAxisLabel: 'Change Number';
		 identifier: #versionNum;
		 rotatedLabels: true;
		 regularAxis;
		 defaultColor: Color blue;
		
		preferredAxisMinY: (changesArray minValue: #changeNum);
		 models: changesArray.
	^ chart open! !

Object subclass: #VersionCriteriaCalculator
	instanceVariableNames: 'classesInfoManager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!VersionCriteriaCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassVersionCriteria: changeCriteria withHistory: classHistory 
	| versionsInfoList |
	versionsInfoList := VersionsList new.
	classHistory versions
		do: [:classVersion | 
			| criteriaValue versionInfo |
			criteriaValue := changeCriteria getNumber: classVersion.
			versionInfo := VersionInfo new.
			versionInfo
				versionNum: ((classVersion name subStrings: ' -')
						at: 1).
			versionInfo value: criteriaValue.
			versionsInfoList add: versionInfo].
	^ versionsInfoList! !

!VersionCriteriaCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesVersionCriteria: changeCriteria fromHismoModel: hismoModel 
	| classesHistories |
	classesHistories := hismoModel allClassHistories
				select: [:each | each last isStub not & each isRemoved not].
	classesHistories
		do: [:classHistory | 
			| className versionsInfo |
			className := classHistory name.
			versionsInfo := self calculateClassVersionCriteria: changeCriteria withHistory: classHistory.
			classesInfoManager addVersionsInfo: versionsInfo forClass: className].
	^ classesInfoManager! !

!VersionCriteriaCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesVersionCriteria: changeCriteria fromMSEFiles: mseFiles 
	| hismoModel processedVersions |
	processedVersions := ((mseFiles first name subStrings: '.')
				at: 1)
				, ' to '
				, ((mseFiles last name subStrings: '.')
						at: 1).
	Transcript show: 'Creating Hismo Model from MSE files for versions: ' , processedVersions;
		 cr.
	hismoModel := self createHismoModelFromMSEFiles: mseFiles.
	Transcript show: 'Calculating classes version criterias from MSE files for versions: ' , processedVersions;
		 cr.
	self calculateClassesVersionCriteria: changeCriteria fromHismoModel: hismoModel.
	hismoModel remove.
	^ classesInfoManager! !

!VersionCriteriaCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesVersionCriteria: changeCriteria fromMSEFiles: mseFilesDirPath numOfFilesPerHismo: numFilesPerHismo 
	| mseFilesDir mseFiles mseFilesForCurrentHismo filesCount |
	Transcript show: 'Calculating classes criterias from the MSE files in ' , mseFilesDirPath;
		 cr.
	mseFilesDir := FileDirectory new setPathName: mseFilesDirPath.
	mseFiles := self sortMSEFiles: mseFilesDir.
	filesCount := 0.
	mseFilesForCurrentHismo := LinkedList new.
	mseFiles
		do: [:mseFile | 
			filesCount := filesCount + 1.
			filesCount <= numFilesPerHismo
				ifTrue: [mseFilesForCurrentHismo add: mseFile]
				ifFalse: [self calculateClassesVersionCriteria: changeCriteria fromMSEFiles: mseFilesForCurrentHismo.
					mseFilesForCurrentHismo removeAll.
					mseFilesForCurrentHismo add: mseFile.
					filesCount := 1]].
	self calculateClassesVersionCriteria: changeCriteria fromMSEFiles: mseFilesForCurrentHismo.
	classesInfoManager removeDeprecatedClasses.
	^ classesInfoManager! !

!VersionCriteriaCalculator methodsFor: 'as yet unclassified' stamp: ''!
createHismoModelFromMSEFiles: mseFiles 
	| hismoModel |
	hismoModel := HismoModelHistory new.
	hismoModel install.
	mseFiles
		do: [:mseFile | 
			| stream mooseModel |
			stream := mseFile readStream.
			mooseModel := MooseModel new importFromMSEStream: stream.
			mooseModel
				name: ((mseFile name subStrings: '.')
						at: 1).
			hismoModel addLastVersionForMooseModel: mooseModel].
	^ hismoModel! !

!VersionCriteriaCalculator methodsFor: 'as yet unclassified' stamp: ''!
initialize
	classesInfoManager := ClassesInfoManager new! !

!VersionCriteriaCalculator methodsFor: 'as yet unclassified' stamp: ''!
sortMSEFiles: mseFilesDir 
	| mseFiles |
	mseFiles := SortedCollection
				sortBlock: [:c1 :c2 | ((c1 name subStrings: '.')
						at: 1) asInteger
						<= ((c2 name subStrings: '.')
								at: 1)].
	mseFiles addAll: mseFilesDir entries.
	^ mseFiles! !

Object subclass: #VersionInfo
	instanceVariableNames: 'versionNum value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!VersionInfo methodsFor: 'as yet unclassified' stamp: ''!
< vInfo 
	^ versionNum < vInfo versionNum! !

!VersionInfo methodsFor: 'as yet unclassified' stamp: ''!
<= vInfo 
	^ versionNum <= vInfo versionNum! !

!VersionInfo methodsFor: 'as yet unclassified' stamp: ''!
= vInfo 
	^ versionNum = vInfo versionNum! !

!VersionInfo methodsFor: 'as yet unclassified' stamp: ''!
> vInfo 
	^ versionNum > vInfo versionNum! !

!VersionInfo methodsFor: 'as yet unclassified' stamp: ''!
>= vInfo 
	^ versionNum >= vInfo versionNum! !

!VersionInfo methodsFor: 'as yet unclassified' stamp: ''!
value
	^ value! !

!VersionInfo methodsFor: 'as yet unclassified' stamp: ''!
value: val 
	value := val! !

!VersionInfo methodsFor: 'as yet unclassified' stamp: ''!
versionNum
	^ versionNum! !

!VersionInfo methodsFor: 'as yet unclassified' stamp: ''!
versionNum: num 
	versionNum := num! !

Object subclass: #VersionsList
	instanceVariableNames: 'versionsInfo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!VersionsList methodsFor: 'as yet unclassified' stamp: ''!
add: vInfo 
	versionsInfo add: vInfo! !

!VersionsList methodsFor: 'as yet unclassified' stamp: ''!
addAll: vInfo 
	versionsInfo addAll: vInfo! !

!VersionsList methodsFor: 'as yet unclassified' stamp: ''!
allVersionNums
	| versionNums |
	versionNums := SortedCollection new.
	versionsInfo
		do: [:versInfo | versionNums add: versInfo versionNum].
	^ versionNums! !

!VersionsList methodsFor: 'as yet unclassified' stamp: ''!
first
	^ versionsInfo first! !

!VersionsList methodsFor: 'as yet unclassified' stamp: ''!
getInfoForVersion: versionNum 
	versionsInfo
		do: [:versionInfo | 
			versionNum = versionInfo versionNum
				ifTrue: [^ versionInfo value].
			nil].
	^ nil! !

!VersionsList methodsFor: 'as yet unclassified' stamp: ''!
getInfoFromVersion: beginVersion toVersion: endVersion 
	| windowVersionsInfo |
	windowVersionsInfo := VersionsList new.
	versionsInfo
		do: [:versionInfo | 
			| versionNum |
			versionNum := versionInfo versionNum.
			versionNum >= beginVersion & (versionNum <= endVersion)
				ifTrue: [windowVersionsInfo add: versionInfo]].
	^ windowVersionsInfo! !

!VersionsList methodsFor: 'as yet unclassified' stamp: ''!
includes: vInfo 
	^ versionsInfo includes: vInfo! !

!VersionsList methodsFor: 'as yet unclassified' stamp: ''!
initialize
	versionsInfo := SortedCollection new! !

!VersionsList methodsFor: 'as yet unclassified' stamp: ''!
isEmpty
	^ versionsInfo isEmpty! !

!VersionsList methodsFor: 'as yet unclassified' stamp: ''!
last
	versionsInfo last! !

!VersionsList methodsFor: 'as yet unclassified' stamp: ''!
size
	^ versionsInfo size! !

!VersionsList methodsFor: 'as yet unclassified' stamp: ''!
versionsInfo
	^ versionsInfo! !

Object subclass: #VolatilityCalculator
	instanceVariableNames: 'sum sum2 num reliabilityFileStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesAccumReliability: classesInfo 
	^ self calculateClassesAccumReliability: classesInfo candidateClassesNum: nil! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesAccumReliability: classesInfo candidateClassesNum: candidateClassesNum 
	| versionsList reliabilityList beginWindowVersion biggestVolClasses windowClassesInfo changedClasses classesVol windowReliability candidatesNum version |
	Transcript show: 'Calculating Accumulated Volatility';
		 cr;
		 cr.
	versionsList := classesInfo getAllExistingVersions.
	reliabilityList := LinkedList new.
	candidatesNum := candidateClassesNum.
	beginWindowVersion := versionsList at: 1.
	2
		to: versionsList size - 1
		do: [:index | 
			version := versionsList at: index.
			reliabilityList
				add: (self
						calculateWindowReliability: classesInfo
						beginWindowVersion: beginWindowVersion
						endWindowVersion: version
						futureVersion: (versionsList at: index + 1)
						candidateClassesNum: candidateClassesNum)].
	^ reliabilityList! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesAccumReliability: classesInfo candidateClassesNum: candidateClassesNum saveToFile: fileName 
	| reliabilityList |
	reliabilityFileStream := FileStream forceNewFileNamed: fileName.
	reliabilityList := self calculateClassesAccumReliability: classesInfo candidateClassesNum: candidateClassesNum.
	reliabilityFileStream close.
	reliabilityFileStream := nil.
	^ reliabilityList! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesAccumReliability: classesInfo saveToFile: fileName 
	^ self
		calculateClassesAccumReliability: classesInfo
		candidateClassesNum: nil
		saveToFile: fileName! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesReliability: classesInfo usingWindow: windowNum 
	^ self
		calculateClassesReliability: classesInfo
		usingWindow: windowNum
		candidateClassesNum: nil! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesReliability: classesInfo usingWindow: windowNum candidateClassesNum: candidateClassesNum 
	| versionsList reliabilityList beginWindowVersionPos endWindowVersion versionCount version |
	Transcript show: 'Calculating Volatility using window ' , windowNum asString;
		 cr;
		 cr.
	versionsList := classesInfo getAllExistingVersions.
	reliabilityList := LinkedList new.
	versionCount := 0.
	beginWindowVersionPos := 1.
	[beginWindowVersionPos <= (versionsList size - windowNum - 1)]
		whileTrue: [versionCount := versionCount + 1.
			version := versionsList at: versionCount.
			versionCount >= (windowNum + 1)
				ifTrue: [reliabilityList
						add: (self
								calculateWindowReliability: classesInfo
								beginWindowVersion: (versionsList at: beginWindowVersionPos)
								endWindowVersion: endWindowVersion
								futureVersion: version
								candidateClassesNum: candidateClassesNum).
					beginWindowVersionPos := beginWindowVersionPos + 1].
			endWindowVersion := version].
	versionCount < (windowNum + 1)
		ifTrue: [reliabilityList
				add: (self
						calculateWindowReliability: classesInfo
						beginWindowVersion: (versionsList at: 1)
						endWindowVersion: (versionsList at: versionsList size - 1)
						futureVersion: (versionsList at: versionsList size)
						candidateClassesNum: candidateClassesNum)].
	^ reliabilityList! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesReliability: classesInfo usingWindow: windowNum candidateClassesNum: candidateClassesNum saveToFile: fileName 
	| reliabilityList |
	reliabilityFileStream := FileStream forceNewFileNamed: fileName.
	reliabilityList := self
				calculateClassesReliability: classesInfo
				usingWindow: windowNum
				candidateClassesNum: candidateClassesNum.
	reliabilityFileStream close.
	reliabilityFileStream := nil.
	^ reliabilityList! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesReliability: classesInfo usingWindow: windowNum saveToFile: fileName 
	^ self
		calculateClassesReliability: classesInfo
		usingWindow: windowNum
		candidateClassesNum: nil
		saveToFile: fileName! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesVolatilities: classesInfoManager 
	| classesVolatilities classesInfo volatility |
	classesVolatilities := SortedCollection new.
	classesInfo := classesInfoManager classesInfo.
	classesInfo
		keysDo: [:className | 
			| classValues classVolatility |
			classValues := classesInfo at: className.
			volatility := self calculateVolatility: classValues.
			classVolatility := VolatilityInfo new.
			classVolatility className: className.
			classVolatility volatility: volatility.
			classesVolatilities add: classVolatility].
	^ classesVolatilities reverse! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateClassesVolatilities: classesInfo saveToFile: fileName 
	| classesVolatilities |
	classesVolatilities := self calculateClassesVolatilities: classesInfo.
	self saveClassesVolatilities: classesVolatilities ToFile: fileName.
	^ classesVolatilities! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateMean
	num > 0
		ifTrue: [^ sum / num].
	^ 0.0! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateReliabilityForCandidates: candidateClasses withChangedClasses: changedClasses 
	| hits |
	hits := 0.
	candidateClasses
		do: [:className | (changedClasses includes: className)
				ifTrue: [hits := hits + 1]].
	^ (hits / candidateClasses size) asFloat printShowingDecimalPlaces: 2! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateStdDev
	^ self calculateVariance sqrt! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateVariance
	| mean |
	mean := self calculateMean.
	num > 0
		ifTrue: [^ sum2 / num - (mean * mean)].
	^ 0.0! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateVolatility: versionsList 
	| revisionsNum volatility stdDev |
	sum := 0.
	sum2 := 0.
	num := 0.
	versionsList versionsInfo
		do: [:versInfo | 
			| value |
			value := versInfo value.
			sum := sum + value.
			sum2 := sum2 + (value * value).
			num := num + 1.
			stdDev := self calculateStdDev].
	revisionsNum := versionsList size.
	^ stdDev * revisionsNum sqrt! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateVolatility: versionsInfo usingWindow: windowNum 
	| windowVersions windowReliance versCount |
	windowVersions := LinkedList new.
	windowReliance := LinkedList new.
	versCount := 0.
	versionsInfo
		do: [:versInfo | 
			| windowVolatility |
			versCount := versCount + 1.
			versCount = windowNum
				ifTrue: [windowVolatility := self calculateVolatility: windowVersions.
					windowReliance add: '()'.
					versCount := 0].
			windowVersions add: versInfo]! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
calculateWindowReliability: classesInfo beginWindowVersion: beginWindowVersion endWindowVersion: endWindowVersion futureVersion: futureVersion candidateClassesNum: candidateClassesNum 
	| windowClassesInfo classesVol changedClasses biggestVolClasses windowReliability candidatesNum |
	candidatesNum := candidateClassesNum.
	windowClassesInfo := classesInfo getClassesInfoFromVersion: beginWindowVersion toVersion: endWindowVersion.
	classesVol := self calculateClassesVolatilities: windowClassesInfo.
	changedClasses := classesInfo getChangedClassesFromVersion: endWindowVersion toVersion: futureVersion.
	candidateClassesNum isNil
		ifTrue: [candidatesNum := changedClasses size].
	biggestVolClasses := self getClassesWithBiggestVolatility: classesVol candidatesNum: candidatesNum.
	changedClasses size > 0 & (biggestVolClasses size > 0)
		ifTrue: [windowReliability := self calculateReliabilityForCandidates: biggestVolClasses withChangedClasses: changedClasses.
			reliabilityFileStream isNil
				ifFalse: [self
						saveReliabilityInfo: windowReliability
						beginVersion: beginWindowVersion
						endVersion: endWindowVersion
						biggestVolClasses: biggestVolClasses
						changedClasses: changedClasses]]
		ifFalse: [windowReliability := 0.0].
	^ windowReliability! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
getClassesWithBiggestVolatility: classesVol candidatesNum: candidatesNum 
	| candidateClasses |
	candidateClasses := LinkedList new.
	1
		to: candidatesNum
		do: [:index | candidateClasses add: (classesVol at: index) className].
	^ candidateClasses! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
initialize
	sum := 0.
	sum2 := 0.
	num := 0.
	reliabilityFileStream := nil! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
saveClassesVolatilities: classesVolatilities ToFile: fileName 
	| stream |
	stream := FileStream forceNewFileNamed: fileName.
	Transcript show: 'Saving Volatility to file ' , fileName , ' ...';
		 cr;
		 cr.
	classesVolatilities
		do: [:classVol | stream nextPutAll: classVol className , ' - ' , classVol volatility asString;
				 cr].
	stream close! !

!VolatilityCalculator methodsFor: 'as yet unclassified' stamp: ''!
saveReliabilityInfo: reliability beginVersion: beginVersion endVersion: endVersion biggestVolClasses: biggestVolClasses changedClasses: changedClasses 
	reliabilityFileStream nextPutAll: 'From ' , beginVersion asString , ' to ' , endVersion asString;
		 cr.
	reliabilityFileStream nextPutAll: 'Biggest Volatility Classes: ' , biggestVolClasses asCommaString;
		 cr;
		 cr.
	reliabilityFileStream nextPutAll: 'Classes that changed: ' , changedClasses asCommaString;
		 cr;
		 cr.
	reliabilityFileStream nextPutAll: 'Reliability: ' , reliability asString;
		 cr;
		 cr;
		 cr! !

Object subclass: #VolatilityInfo
	instanceVariableNames: 'className volatility'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!VolatilityInfo methodsFor: 'as yet unclassified' stamp: ''!
< volInfo 
	^ volatility < volInfo volatility! !

!VolatilityInfo methodsFor: 'as yet unclassified' stamp: ''!
<= volInfo 
	^ volatility <= volInfo volatility! !

!VolatilityInfo methodsFor: 'as yet unclassified' stamp: ''!
= volInfo 
	^ volatility = volInfo volatility! !

!VolatilityInfo methodsFor: 'as yet unclassified' stamp: ''!
> volInfo 
	^ volatility > volInfo volatility! !

!VolatilityInfo methodsFor: 'as yet unclassified' stamp: ''!
>= volInfo 
	^ volatility >= volInfo volatility! !

!VolatilityInfo methodsFor: 'as yet unclassified' stamp: ''!
className
	^ className! !

!VolatilityInfo methodsFor: 'as yet unclassified' stamp: ''!
className: cName 
	className := cName! !

!VolatilityInfo methodsFor: 'as yet unclassified' stamp: ''!
volatility
	^ volatility! !

!VolatilityInfo methodsFor: 'as yet unclassified' stamp: ''!
volatility: vol 
	volatility := vol! !
